@@ -13,6 +13,7 @@
 #include <KFormat>
 #include <KLocalizedString>
 #include <KProtocolInfo>
+#include <KShell>
 
 #include <QDebug>
 #include <QTimer>
@@ -32,7 +33,7 @@ namespace
 QVariant intersect(const QVariant &v1, const QVariant &v2)
 {
     if (!v1.isValid() || !v2.isValid()) {
-        return QVariant();
+        return {};
     }
 
     // List and String
@@ -71,7 +72,7 @@ QVariant intersect(const QVariant &v1, const QVariant &v2)
         return v1;
     }
 
-    return QVariant();
+    return {};
 }
 
 /**
@@ -117,7 +118,7 @@ void FileMetaDataProvider::totalPropertyAndInsert(const QString &prop, const QLi
 
 void FileMetaDataProvider::slotFileFetchFinished(KJob *job)
 {
-    FileFetchJob *fetchJob = static_cast<FileFetchJob *>(job);
+    auto fetchJob = static_cast<FileFetchJob *>(job);
     QList<QVariantMap> files = fetchJob->data();
 
     Q_ASSERT(!files.isEmpty());
@@ -181,6 +182,9 @@ void FileMetaDataProvider::insertSingleFileBasicData()
         if (item.isLink()) {
             m_data.insert(QStringLiteral("kfileitem#linkDest"), item.linkDest());
         }
+        if (item.entry().contains(KIO::UDSEntry::UDS_TARGET_URL)) {
+            m_data.insert(QStringLiteral("kfileitem#targetUrl"), KShell::tildeCollapse(item.targetUrl().toDisplayString(QUrl::PreferLocalFile)));
+        }
         QDateTime modificationTime = item.time(KFileItem::ModificationTime);
         if (modificationTime.isValid()) {
             m_data.insert(QStringLiteral("kfileitem#modified"), modificationTime);
@@ -231,7 +235,7 @@ void FileMetaDataProvider::insertFilesListBasicData()
     // If all directories
     Q_ASSERT(m_fileItems.count() > 1);
     bool allDirectories = true;
-    for (const KFileItem &item : qAsConst(m_fileItems)) {
+    for (const KFileItem &item : std::as_const(m_fileItems)) {
         allDirectories &= item.isDir();
         if (!allDirectories) {
             break;
@@ -242,7 +246,7 @@ void FileMetaDataProvider::insertFilesListBasicData()
         int count = 0;
         int hiddenCount = 0;
         bool isSizeKnown = true;
-        for (const KFileItem &item : qAsConst(m_fileItems)) {
+        for (const KFileItem &item : std::as_const(m_fileItems)) {
             isSizeKnown = item.isLocalFile() && !item.isSlow();
             if (!isSizeKnown) {
                 return;
@@ -267,7 +271,7 @@ void FileMetaDataProvider::insertFilesListBasicData()
     } else {
         // Calculate the size of all items
         quint64 totalSize = 0;
-        for (const KFileItem &item : qAsConst(m_fileItems)) {
+        for (const KFileItem &item : std::as_const(m_fileItems)) {
             if (!item.isDir() && !item.isLink()) {
                 totalSize += item.size();
             }
@@ -301,7 +305,9 @@ void FileMetaDataProvider::insertCommonData(const QList<QVariantMap> &files)
     QList<QVariantMap> propertyList;
     for (const QVariantMap &fileData : files) {
         propertyList << fileData;
-        allProperties.unite(fileData.uniqueKeys().toSet());
+        auto uniqueValues = fileData.keys();
+        uniqueValues.erase(std::unique(uniqueValues.begin(), uniqueValues.end()), uniqueValues.end());
+        allProperties += QSet<QString>(uniqueValues.begin(), uniqueValues.end());
     }
 
     // Special handling for certain properties
@@ -310,8 +316,8 @@ void FileMetaDataProvider::insertCommonData(const QList<QVariantMap> &files)
     totalPropertyAndInsert(QStringLiteral("wordCount"), propertyList, allProperties);
     totalPropertyAndInsert(QStringLiteral("lineCount"), propertyList, allProperties);
 
-    for (const QString &propUri : qAsConst(allProperties)) {
-        for (const QVariantMap &map : qAsConst(propertyList)) {
+    for (const QString &propUri : std::as_const(allProperties)) {
+        for (const QVariantMap &map : std::as_const(propertyList)) {
             QVariantMap::const_iterator it = map.find(propUri);
             if (it == map.constEnd()) {
                 m_data.remove(propUri);
@@ -341,9 +347,7 @@ FileMetaDataProvider::FileMetaDataProvider(QObject *parent)
 {
 }
 
-FileMetaDataProvider::~FileMetaDataProvider()
-{
-}
+FileMetaDataProvider::~FileMetaDataProvider() = default;
 
 void FileMetaDataProvider::setFileItem()
 {
@@ -389,7 +393,7 @@ void FileMetaDataProvider::setFileItems()
     urls.reserve(m_fileItems.size());
     // Only extract data from indexed files,
     // it would be too expensive otherwise.
-    for (const KFileItem &item : qAsConst(m_fileItems)) {
+    for (const KFileItem &item : std::as_const(m_fileItems)) {
         const QUrl url = item.targetUrl();
         if (url.isLocalFile() && !item.isSlow()) {
             urls << url.toLocalFile();
@@ -401,7 +405,7 @@ void FileMetaDataProvider::setFileItems()
         // Editing only if all URLs are local
         bool canEdit = (urls.size() == m_fileItems.size());
 
-        FileFetchJob *job = new FileFetchJob(urls, canEdit, FileFetchJob::UseRealtimeIndexing::Disabled, this);
+        auto job = new FileFetchJob(urls, canEdit, FileFetchJob::UseRealtimeIndexing::Disabled, this);
         connect(job, &FileFetchJob::finished, this, &FileMetaDataProvider::slotFileFetchFinished);
         job->start();
 
@@ -444,6 +448,7 @@ QString FileMetaDataProvider::label(const QString &metaDataLabel) const
         {QStringLiteral("kfileitem#hiddenItems"), i18nc("@label", "Hidden items")},
         {QStringLiteral("kfileitem#type"), i18nc("@label", "Type")},
         {QStringLiteral("kfileitem#linkDest"), i18nc("@label", "Link to")},
+        {QStringLiteral("kfileitem#targetUrl"), i18nc("@label", "Points to")},
         {QStringLiteral("tags"), i18nc("@label", "Tags")},
         {QStringLiteral("rating"), i18nc("@label", "Rating")},
         {QStringLiteral("userComment"), i18nc("@label", "Comment")},
@@ -466,8 +471,11 @@ QString FileMetaDataProvider::label(const QString &metaDataLabel) const
 
             // Have to construct a dummy URL for KProtocolInfo::extraFields...
             QUrl url;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
             url.setScheme(protocol.toString());
-
+#else
+            url.setScheme(protocol);
+#endif
             const auto extraFields = KProtocolInfo::extraFields(url);
             auto field = extraFields.value(extraNumber);
             if (field.type != KProtocolInfo::ExtraField::Invalid) {
